//+------------------------------------------------------------------+
//|                                             FrankieSRLinesEA.mq4 |
//|                                                                  |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
//#property 
//#property 
//#define   
#property strict
//+------------------------------------------------------------------+
#include <stdlib.mqh>
#include <stderror.mqh>
//+------------------------------------------------------------------+
//| Input Parameters Definition                                      |
//+------------------------------------------------------------------+
input string                  hint1                         = "===== trade entry settings =====";
input bool                    AllowLong                     = true;
input bool                    AllowShort                    = true;
input bool                    AutoLot                       = false;
input double                  Risk                          = 1;
input double                  FixedLot                      = 0.1;

input bool                    AllowInitialEntries           = true;
input double                  BuyEntryAddPoints             = 50;
input double                  SellEntryAddPoints            = 50;
input double                  BuyStopLoss                   = 150;
input double                  BuyTakeProfit                 = 100;
input double                  SellStopLoss                  = 150;
input double                  SellTakeProfit                = 100;

input bool                    AllowLevelEntries             = true;
input double                  LevelsBuyEntryAddPoints       = 0;
input int                     LevelsBuySLPoints             = 100;
input int                     LevelsBuyTPPoints             = 100;

input double                  LevelsSellEntryAddPoints      = 0;
input int                     LevelsSellSLPoints            = 100;
input int                     LevelsSellTPPoints            = 100;

input string                  hint2                         = "===== indicator settings =====";
input string                  MidnightTime                  = "00:00";
input color                   MidnightColor                 = clrBrown;
input color                   MainLineColor                 = clrYellow;
input color                   RLineColor                    = clrGreen;
input color                   SLineColor                    = clrRed;
input int                     SRPoints                      = 125;
input int                     SRLevelCount                  = 17;
input bool                    ShowAlert                     = true;
input bool                    PopupAlert                    = true;
input bool                    PushAlert                     = false;
input bool                    MailAlert                     = false;
input int                     AlertStartPoints              = 345;
input int                     AlertLevelPoints              = 250;
input int                     AlertLevelCount               = 8;

input string                  hint3                         = "===== other settings =====";
input int                     MagicNumber                   = 47722;
input string                  TradeComment                  = "";
//+------------------------------------------------------------------+
//| Local Parameters Definition                                      |
//+------------------------------------------------------------------+
string lbl = "FrankieSRLinesEA.", gbl;
int DecimalPoints;
bool _res;
datetime lastAlert;
double lastAlertLevel;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
  //------------------------------------------------------------------
  while (MarketInfo(Symbol(), MODE_LOTSTEP)==0) Sleep(2000);
  DecimalPoints = 1;
  double x = MarketInfo(Symbol(), MODE_LOTSTEP) * 10;
  while (x < 1)
  { x *= 10; DecimalPoints += 1; }
  //------------------------------------------------------------------
  gbl = lbl+Symbol()+"."+IntegerToString(MagicNumber)+".";
  if (IsTesting()) { gbl = "B."+gbl; GlobalVariablesDeleteAll(gbl); }
  //------------------------------------------------------------------
  lastAlert = 0; lastAlertLevel = 0;
  EventSetTimer(1);
  //------------------------------------------------------------------
  return (INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
  //------------------------------------------------------------------
  EventKillTimer();
  if (reason == REASON_REMOVE) GlobalVariablesDeleteAll(gbl);
  ObjectsDeleteAll(0, lbl);
  Comment("");
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
void OnTimer()
{
  //------------------------------------------------------------------
  RefreshRates();
  OnTick();
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
void OnTick()
{
  //------------------------------------------------------------------
  if (!IsConnected() || AccountNumber() == 0 || Bars <= 30 || ArraySize(Time) == 0) return;
  //------------------------------------------------------------------
  datetime midnight = StrToTime(TimeToStr(iTime(NULL, PERIOD_D1, 0), TIME_DATE)+" "+MidnightTime);
  int bar = iBarShift(NULL, PERIOD_M1, midnight);
  double upperPrice = 0, lowerPrice = 0;
  if (iTime(NULL, PERIOD_M1, bar)==midnight)
  {
    DrawVLine(lbl+"Midnight", iTime(NULL, PERIOD_M1, bar), iOpen(NULL, PERIOD_M1, bar), MidnightColor, STYLE_DOT, 1, true);
    double upper = iHigh(NULL, PERIOD_M1, bar); upperPrice = upper;
    double lower = iLow(NULL, PERIOD_M1, bar); lowerPrice = lower;
    DrawLine(lbl+"High.0", midnight, upper, MainLineColor, STYLE_SOLID, 1);
    DrawLine(lbl+"Low.0", midnight, lower, MainLineColor, STYLE_SOLID, 1);
    ENUM_LINE_STYLE lineStyle = STYLE_DASH;
    for (int i = 1; i <= SRLevelCount; i++)
    {
      upper += SRPoints * _Point;
      DrawLine(lbl+"High."+IntegerToString(i), midnight, upper, RLineColor, lineStyle, 1);
      if (lineStyle == STYLE_SOLID) lineStyle = STYLE_DASH; else lineStyle = STYLE_SOLID;
    }
    lineStyle = STYLE_DASH;
    for (int i = 1; i <= SRLevelCount; i++)
    {
      lower -= SRPoints * _Point;
      DrawLine(lbl+"Low."+IntegerToString(i), midnight, lower, SLineColor, lineStyle, 1);
      if (lineStyle == STYLE_SOLID) lineStyle = STYLE_DASH; else lineStyle = STYLE_SOLID;
    }
    if (ShowAlert)
    {
      if (lastAlert == 0)
      {
        lastAlertLevel = Bid;
        lastAlert = midnight;
      }
      else if (lastAlert < midnight)
      {
        lastAlert = midnight;
        lastAlertLevel = Bid;
      }
      upper = iHigh(NULL, PERIOD_M1, bar) + AlertStartPoints*_Point;
      lower = iLow(NULL, PERIOD_M1, bar) - AlertStartPoints*_Point;
      string msg;
      for (int i = 0; i <= AlertLevelCount; i++)
      {
        if (NormalizeDouble(lastAlertLevel, _Digits)<NormalizeDouble(upper, _Digits) && NormalizeDouble(Bid, _Digits)>=NormalizeDouble(upper, _Digits))
        {
          lastAlertLevel = upper;
          msg = TimeToStr(TimeCurrent(), TIME_SECONDS)+": "+Symbol()+" level "+IntegerToString(i+1)+"("+DoubleToStr(upper, _Digits)+") touched";
          if (PopupAlert) Alert(msg);
          if (PushAlert) SendNotification(msg);
          if (MailAlert) SendMail(msg, msg);
          break;
        }
        upper += AlertLevelPoints*_Point;
      }
      for (int i = 0; i <= AlertLevelCount; i++)
      {
        if (NormalizeDouble(lastAlertLevel, _Digits)>NormalizeDouble(lower, _Digits) && NormalizeDouble(Bid, _Digits)<=NormalizeDouble(lower, _Digits))
        {
          lastAlertLevel = lower;
          msg = TimeToStr(TimeCurrent(), TIME_SECONDS)+": "+Symbol()+" level -"+IntegerToString(i+1)+"("+DoubleToStr(lower, _Digits)+") touched";
          if (PopupAlert) Alert(msg);
          if (PushAlert) SendNotification(msg);
          if (MailAlert) SendMail(msg, msg);
          break;
        }
        lower -= AlertLevelPoints*_Point;
      }
    }
  }
  //------------------------------------------------------------------
  if (AllowInitialEntries && bar == 0 && upperPrice != 0 && lowerPrice != 0)
  {
    if (AllowLong && DoesSignalCatched(OP_BUY, iTime(NULL, PERIOD_M1, 0), 0)==-1) SetOrder(OP_BUYLIMIT, lowerPrice-PipsToPoints(BuyEntryAddPoints), 0);
    if (AllowShort && DoesSignalCatched(OP_SELL, iTime(NULL, PERIOD_M1, 0), 0)==-1) SetOrder(OP_SELLLIMIT, upperPrice+PipsToPoints(SellEntryAddPoints), 0);
  }
  //------------------------------------------------------------------
  if (AllowLevelEntries && AllowLong)
  {
    for (int i = 2; i < SRLevelCount; i+=2)
    {
      double entry = NormalizeDouble(ObjectGetDouble(0, lbl+"High."+IntegerToString(i), OBJPROP_PRICE, 0), _Digits)+PipsToPoints(LevelsBuyEntryAddPoints);
      double sl = entry - LevelsBuySLPoints*_Point;
      double tp = entry + LevelsBuyTPPoints*_Point;
      double nlevel = NormalizeDouble(ObjectGetDouble(0, lbl+"High."+IntegerToString(i+1), OBJPROP_PRICE, 0), _Digits);
      double plevel = NormalizeDouble(ObjectGetDouble(0, lbl+"High."+IntegerToString(i-1), OBJPROP_PRICE, 0), _Digits);
      if (Bid >= nlevel)
      {
        int last = GetLastOrder(OP_BUY, midnight, i);
        if (last == -1)
        {
          int ticket = SetOrder(OP_BUYLIMIT, entry, sl, tp, i);
        }
        else if (last != -1 && OrderCloseTime()!=0)
        {
          int b = iBarShift(NULL, PERIOD_M1, OrderCloseTime());
          if (b>0)
          {
            int l = iLowest(NULL, PERIOD_M1, MODE_LOW, b+1, 0);
            if (iLow(NULL, PERIOD_M1, l)<=plevel)
            {
              int ticket = SetOrder(OP_BUYLIMIT, entry, sl, tp, i);
            }
          }
        }
      }
    }
  }
  if (AllowLevelEntries && AllowShort)
  {
    for (int i = 2; i < SRLevelCount; i+=2)
    {
      double entry = NormalizeDouble(ObjectGetDouble(0, lbl+"Low."+IntegerToString(i), OBJPROP_PRICE, 0), _Digits)-PipsToPoints(LevelsSellEntryAddPoints);
      double sl = entry + LevelsSellSLPoints*_Point;
      double tp = entry - LevelsSellTPPoints*_Point;
      double nlevel = NormalizeDouble(ObjectGetDouble(0, lbl+"Low."+IntegerToString(i+1), OBJPROP_PRICE, 0), _Digits);
      double plevel = NormalizeDouble(ObjectGetDouble(0, lbl+"Low."+IntegerToString(i-1), OBJPROP_PRICE, 0), _Digits);
      if (Bid <= nlevel)
      {
        int last = GetLastOrder(OP_SELL, midnight, i);
        if (last == -1)
        {
          int ticket = SetOrder(OP_SELLLIMIT, entry, sl, tp, i);
        }
        else if (last != -1 && OrderCloseTime()!=0)
        {
          int b = iBarShift(NULL, PERIOD_M1, OrderCloseTime());
          if (b>0)
          {
            int h = iHighest(NULL, PERIOD_M1, MODE_HIGH, b+1, 0);
            if (iHigh(NULL, PERIOD_M1, h)>=plevel)
            {
              int ticket = SetOrder(OP_SELLLIMIT, entry, sl, tp, i);
            }
          }
        }
      }
    }
  }
  //------------------------------------------------------------------
  ManageOrders(midnight);
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
int SetOrder(int type, double price, int level)
{
  double sl = 0, tp = 0;
  color col = clrNONE;
  if (type == OP_BUYLIMIT)
  {
    col = clrBlue;
    if (BuyStopLoss != 0) sl = price - PipsToPoints(BuyStopLoss);
    if (BuyTakeProfit != 0) tp = price + PipsToPoints(BuyTakeProfit);
  }
  if (type == OP_SELLLIMIT)
  {
    col = clrRed;
    if (SellStopLoss != 0) sl = price + PipsToPoints(SellStopLoss);
    if (SellTakeProfit != 0) tp = price - PipsToPoints(SellTakeProfit);
  }
  double lot = FixedLot;
  if (AutoLot && sl != 0)
  {
    double pv = SymbolInfoDouble(Symbol(), SYMBOL_TRADE_TICK_VALUE)/SymbolInfoDouble(Symbol(), SYMBOL_TRADE_TICK_SIZE);
    lot = NormalizeDouble(Risk*AccountEquity() / (100*MathAbs(price-sl)*pv), DecimalPoints);
    if (lot < MarketInfo(Symbol(), MODE_MINLOT)) lot = MarketInfo(Symbol(), MODE_MINLOT);
    if (lot > MarketInfo(Symbol(), MODE_MAXLOT)) lot = MarketInfo(Symbol(), MODE_MAXLOT);
  }
  int ticket = OrderSend(Symbol(), type, lot, price, 0, sl, tp, TradeComment, MagicNumber, 0, col);
  int err = GetLastError();
  if (ticket == -1 && err > 0)
    Print("err=", IntegerToString(err), ", type=" + GetOrderTypeName(type), ", price=", DoubleToStr(price, _Digits), ", lot=", DoubleToStr(lot, DecimalPoints), ", ", ErrorDescription(err));
  else if (OrderSelect(ticket, SELECT_BY_TICKET))
  {
    GlobalVariableSet(gbl+IntegerToString(ticket)+".Level", level);
  }
  return (ticket);
}
//+------------------------------------------------------------------+
int SetOrder(int type, double price, double sl, double tp, int level)
{
  color col = clrNONE;
  if (type == OP_BUYLIMIT)
  {
    col = clrBlue;
  }
  if (type == OP_SELLLIMIT)
  {
    col = clrRed;
  }
  double lot = FixedLot;
  if (AutoLot && sl != 0)
  {
    double pv = SymbolInfoDouble(Symbol(), SYMBOL_TRADE_TICK_VALUE)/SymbolInfoDouble(Symbol(), SYMBOL_TRADE_TICK_SIZE);
    lot = NormalizeDouble(Risk*AccountEquity() / (100*MathAbs(price-sl)*pv), DecimalPoints);
    if (lot < MarketInfo(Symbol(), MODE_MINLOT)) lot = MarketInfo(Symbol(), MODE_MINLOT);
    if (lot > MarketInfo(Symbol(), MODE_MAXLOT)) lot = MarketInfo(Symbol(), MODE_MAXLOT);
  }
  int ticket = OrderSend(Symbol(), type, lot, price, 0, sl, tp, TradeComment, MagicNumber, 0, col);
  int err = GetLastError();
  if (ticket == -1 && err > 0)
    Print("err=", IntegerToString(err), ", type=" + GetOrderTypeName(type), ", lot=", DoubleToStr(lot, DecimalPoints), ", price=", DoubleToStr(price, _Digits), ", sl=", DoubleToStr(sl, _Digits), ", tp=", DoubleToStr(tp, _Digits), ", ", ErrorDescription(err));
  else if (OrderSelect(ticket, SELECT_BY_TICKET))
  {
    GlobalVariableSet(gbl+IntegerToString(ticket)+".Level", level);
  }
  return (ticket);
}
//+------------------------------------------------------------------+
void ManageOrders(datetime midStart)
{
  //------------------------------------------------------------------
  for (int i = OrdersTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
    if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
    {
      //------------------------------------------------------------------
      int ticket = OrderTicket();
      if (OrderType()==OP_BUYLIMIT && OrderOpenTime()<midStart && TimeCurrent()>=midStart)
      {
        Print("try to delete expired order");
        _res = OrderDelete(OrderTicket(), clrBlue);
        continue;
      }
      if (OrderType()==OP_SELLLIMIT && OrderOpenTime()<midStart && TimeCurrent()>=midStart)
      {
        Print("try to delete expired order");
        _res = OrderDelete(OrderTicket(), clrRed);
        continue;
      }
      if (OrderType()==OP_BUY && OrderOpenTime()<midStart && TimeCurrent()>=midStart)
      {
        Print("try to close expired order");
        _res = OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 20, clrBlue);
        continue;
      }
      if (OrderType()==OP_SELL && OrderOpenTime()<midStart && TimeCurrent()>=midStart)
      {
        Print("try to close expired order");
        _res = OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 20, clrRed);
        continue;
      }
      //------------------------------------------------------------------
    }
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
int DoesSignalCatched(int type, datetime from, int level)
{
  int type2 = -1; if (type==OP_BUY) type2 = OP_BUYLIMIT; else type2 = OP_SELLLIMIT;
  for (int i = OrdersTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
    if (OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber && (OrderType()==type || OrderType()==type2) && OrderOpenTime()>=from)
    if (level==-1 || (int)GlobalVariableGet(gbl+IntegerToString(OrderTicket())+".Level")==level)
      return (OrderTicket());
  for (int i = OrdersHistoryTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY))
    if (OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber && (OrderType()==type || OrderType()==type2) && OrderOpenTime()>=from)
    if (level==-1 || (int)GlobalVariableGet(gbl+IntegerToString(OrderTicket())+".Level")==level)
      return (OrderTicket());
  return (-1);
}
//+------------------------------------------------------------------+
int GetLastOrder(int type, datetime from, int level)
{
  int type2 = -1; if (type==OP_BUY) type2 = OP_BUYLIMIT; else type2 = OP_SELLLIMIT;
  for (int i = OrdersTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
    if (OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber && (OrderType()==type || OrderType()==type2) && OrderOpenTime()>=from)
    if (level==-1 || (int)GlobalVariableGet(gbl+IntegerToString(OrderTicket())+".Level")==level)
      return (OrderTicket());
  for (int i = OrdersHistoryTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY))
    if (OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber && (OrderType()==type || OrderType()==type2) && OrderOpenTime()>=from)
    if (level==-1 || (int)GlobalVariableGet(gbl+IntegerToString(OrderTicket())+".Level")==level)
      return (OrderTicket());
  return (-1);
}
//+------------------------------------------------------------------+
double PipsToPoints(double pips)
{
  //------------------------------------------------------------------
  return (pips * _Point);
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
double PointsToPips(double points)
{
  //------------------------------------------------------------------
  return (points / _Point);
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
string GetOrderTypeName(int type)
{
  if (type == OP_BUY) return ("Buy");
  else if (type == OP_BUYSTOP) return ("Buy Stop");
  else if (type == OP_BUYLIMIT) return ("Buy Limit");
  else if (type == OP_SELL) return ("Sell");
  else if (type == OP_SELLSTOP) return ("Sell Stop");
  else if (type == OP_SELLLIMIT) return ("Sell Limit");
  else return("None");
}
//+------------------------------------------------------------------+
string GetTimeFrameName(int tf)
{
  switch (tf)
  {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    case 0: return(GetTimeFrameName(Period()));
  }
  return ("");
}
//+------------------------------------------------------------------+
void DrawLine(string name, datetime time, double price, color col, int style, int width)
{
  if (ObjectFind(0, name) == -1)
  {
    ObjectCreate(0, name, OBJ_HLINE, 0, time, price);
    ObjectSetInteger(0, name, OBJPROP_RAY, false);
    ObjectSetInteger(0, name, OBJPROP_BACK, false);
    ObjectSetInteger(0, name, OBJPROP_SELECTED, false);
    ObjectSetInteger(0, name, OBJPROP_SELECTABLE, true);
    ObjectSetInteger(0, name, OBJPROP_HIDDEN, true);
  }
  if (ObjectGetInteger(0, name, OBJPROP_COLOR) != col) ObjectSetInteger(0, name, OBJPROP_COLOR, col);
  if (ObjectGetInteger(0, name, OBJPROP_WIDTH) != width) ObjectSetInteger(0, name, OBJPROP_WIDTH, width);
  if (ObjectGetInteger(0, name, OBJPROP_STYLE) != style) ObjectSetInteger(0, name, OBJPROP_STYLE, style);
  if (ObjectGetInteger(0, name, OBJPROP_TIME, 0) != time) ObjectSetInteger(0, name, OBJPROP_TIME, 0, time);
  if (ObjectGetInteger(0, name, OBJPROP_TIME, 1) != time) ObjectSetInteger(0, name, OBJPROP_TIME, 1, time);
  if (NormalizeDouble(ObjectGetDouble(0, name, OBJPROP_PRICE, 0), _Digits) != price) ObjectSetDouble(0, name, OBJPROP_PRICE, 0, price);
  if (NormalizeDouble(ObjectGetDouble(0, name, OBJPROP_PRICE, 1), _Digits) != price) ObjectSetDouble(0, name, OBJPROP_PRICE, 1, price);
}
//+------------------------------------------------------------------+
void DrawVLine(string name, datetime time, double price, color col, int style, int width, bool back)
{
  if (ObjectFind(0, name) == -1)
  {
    ObjectCreate(0, name, OBJ_VLINE, 0, time, price);
    ObjectSetInteger(0, name, OBJPROP_RAY, false);
    ObjectSetInteger(0, name, OBJPROP_BACK, back);
    ObjectSetInteger(0, name, OBJPROP_SELECTABLE, false);
    ObjectSetInteger(0, name, OBJPROP_HIDDEN, true);
  }
  if (ObjectGetInteger(0, name, OBJPROP_COLOR) != col) ObjectSetInteger(0, name, OBJPROP_COLOR, col);
  if (ObjectGetInteger(0, name, OBJPROP_WIDTH) != width) ObjectSetInteger(0, name, OBJPROP_WIDTH, width);
  if (ObjectGetInteger(0, name, OBJPROP_STYLE) != style) ObjectSetInteger(0, name, OBJPROP_STYLE, style);
  if (ObjectGetInteger(0, name, OBJPROP_TIME, 0) != time) ObjectSetInteger(0, name, OBJPROP_TIME, 0, time);
}
//+------------------------------------------------------------------+
