//+------------------------------------------------------------------+
//|                                               FrankieSRLines.mq4 |
//|                                                                  |
//|                                                                  |
//|                                                                   |
//+------------------------------------------------------------------+
//#property
//#property 
//#property 
#property strict
#property indicator_chart_window
#property indicator_buffers 1
//+------------------------------------------------------------------+
//| Input Parameters Definition                                      |
//+------------------------------------------------------------------+
input string                  hint1                         = "===== general setting =====";
input string                  MidnightTime                  = "00:00";
input color                   MainLineColor                 = clrYellow;
input color                   RLineColor                    = clrGreen;
input color                   SLineColor                    = clrRed;
input int                     SRPoints                      = 125;
input int                     SRLevelCount                  = 17;
input bool                    ShowAlert                     = true;
input bool                    PopupAlert                    = true;
input bool                    PushAlert                     = false;
input bool                    MailAlert                     = false;
input int                     AlertStartPoints              = 345;
input int                     AlertLevelPoints              = 250;
input int                     AlertLevelCount               = 8;
//+------------------------------------------------------------------+
//| Local Parameters Definition                                      |
//+------------------------------------------------------------------+
string lbl = "FrankieSRLines.";
datetime lastAlert;
double lastAlertLevel;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
  //------------------------------------------------------------------
  lastAlert = 0; lastAlertLevel = 0;
  //------------------------------------------------------------------
  return (INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
  //------------------------------------------------------------------
  ObjectsDeleteAll(0, lbl);
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate (const int rates_total,      // size of input time series
                 const int prev_calculated,  // bars handled in previous call
                 const datetime& time[],     // Time
                 const double& open[],       // Open
                 const double& high[],       // High
                 const double& low[],        // Low
                 const double& close[],      // Close
                 const long& tick_volume[],  // Tick Volume
                 const long& volume[],       // Real Volume
                 const int& _spread[]        // Spread
)
{
  //------------------------------------------------------------------
  datetime midnight = StrToTime(TimeToStr(iTime(NULL, PERIOD_D1, 0), TIME_DATE)+" "+MidnightTime);
  int bar = iBarShift(NULL, PERIOD_M1, midnight);
  if (iTime(NULL, PERIOD_M1, bar)==midnight)
  {
    double upper = iHigh(NULL, PERIOD_M1, bar);
    double lower = iLow(NULL, PERIOD_M1, bar);
    DrawLine(lbl+"High.0", midnight, upper, MainLineColor, STYLE_SOLID, 1);
    DrawLine(lbl+"Low.0", midnight, lower, MainLineColor, STYLE_SOLID, 1);
    ENUM_LINE_STYLE lineStyle = STYLE_DASH;
    for (int i = 1; i <= SRLevelCount; i++)
    {
      upper += SRPoints * _Point;
      DrawLine(lbl+"High."+IntegerToString(i), midnight, upper, RLineColor, lineStyle, 1);
      if (lineStyle == STYLE_SOLID) lineStyle = STYLE_DASH; else lineStyle = STYLE_SOLID;
    }
    lineStyle = STYLE_DASH;
    for (int i = 1; i <= SRLevelCount; i++)
    {
      lower -= SRPoints * _Point;
      DrawLine(lbl+"Low."+IntegerToString(i), midnight, lower, SLineColor, lineStyle, 1);
      if (lineStyle == STYLE_SOLID) lineStyle = STYLE_DASH; else lineStyle = STYLE_SOLID;
    }
    if (ShowAlert)
    {
      if (lastAlert == 0)
      {
        lastAlertLevel = Bid;
        lastAlert = midnight;
      }
      else if (lastAlert < midnight)
      {
        lastAlert = midnight;
        lastAlertLevel = Bid;
      }
      upper = iHigh(NULL, PERIOD_M1, bar) + AlertStartPoints*_Point;
      lower = iLow(NULL, PERIOD_M1, bar) - AlertStartPoints*_Point;
      string msg;
      for (int i = 0; i <= AlertLevelCount; i++)
      {
        if (NormalizeDouble(lastAlertLevel, _Digits)<NormalizeDouble(upper, _Digits) && NormalizeDouble(Bid, _Digits)>=NormalizeDouble(upper, _Digits))
        {
          lastAlertLevel = upper;
          msg = TimeToStr(TimeCurrent(), TIME_SECONDS)+": "+Symbol()+" level "+IntegerToString(i+1)+"("+DoubleToStr(upper, _Digits)+") touched";
          if (PopupAlert) Alert(msg);
          if (PushAlert) SendNotification(msg);
          if (MailAlert) SendMail(msg, msg);
          break;
        }
        upper += AlertLevelPoints*_Point;
      }
      for (int i = 0; i <= AlertLevelCount; i++)
      {
        if (NormalizeDouble(lastAlertLevel, _Digits)>NormalizeDouble(lower, _Digits) && NormalizeDouble(Bid, _Digits)<=NormalizeDouble(lower, _Digits))
        {
          lastAlertLevel = lower;
          msg = TimeToStr(TimeCurrent(), TIME_SECONDS)+": "+Symbol()+" level -"+IntegerToString(i+1)+"("+DoubleToStr(lower, _Digits)+") touched";
          if (PopupAlert) Alert(msg);
          if (PushAlert) SendNotification(msg);
          if (MailAlert) SendMail(msg, msg);
          break;
        }
        lower -= AlertLevelPoints*_Point;
      }
    }
  }
  //------------------------------------------------------------------
  return (rates_total);
}
//+------------------------------------------------------------------+
double PipsToPoints(double pips)
{
  //------------------------------------------------------------------
  return (pips * _Point);
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
void DrawLine(string name, datetime time, double price, color col, int style, int width)
{
  if (ObjectFind(0, name) == -1)
  {
    ObjectCreate(0, name, OBJ_HLINE, 0, time, price);
    ObjectSetInteger(0, name, OBJPROP_RAY, false);
    ObjectSetInteger(0, name, OBJPROP_BACK, false);
    ObjectSetInteger(0, name, OBJPROP_SELECTED, false);
    ObjectSetInteger(0, name, OBJPROP_SELECTABLE, true);
    ObjectSetInteger(0, name, OBJPROP_HIDDEN, true);
  }
  if (ObjectGetInteger(0, name, OBJPROP_COLOR) != col) ObjectSetInteger(0, name, OBJPROP_COLOR, col);
  if (ObjectGetInteger(0, name, OBJPROP_WIDTH) != width) ObjectSetInteger(0, name, OBJPROP_WIDTH, width);
  if (ObjectGetInteger(0, name, OBJPROP_STYLE) != style) ObjectSetInteger(0, name, OBJPROP_STYLE, style);
  if (ObjectGetInteger(0, name, OBJPROP_TIME, 0) != time) ObjectSetInteger(0, name, OBJPROP_TIME, 0, time);
  if (ObjectGetInteger(0, name, OBJPROP_TIME, 1) != time) ObjectSetInteger(0, name, OBJPROP_TIME, 1, time);
  if (NormalizeDouble(ObjectGetDouble(0, name, OBJPROP_PRICE, 0), _Digits) != price) ObjectSetDouble(0, name, OBJPROP_PRICE, 0, price);
  if (NormalizeDouble(ObjectGetDouble(0, name, OBJPROP_PRICE, 1), _Digits) != price) ObjectSetDouble(0, name, OBJPROP_PRICE, 1, price);
}
//+------------------------------------------------------------------+
